require 'rails_helper'

module Mks
  module Maintenance
    RSpec.describe MaintenanceCategoriesController, type: :controller do
      routes { Mks::Maintenance::Engine.routes }

      before(:each) do
        u = create(:user)
        token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
        request.headers['Authorization'] = "Bearer #{token}"
      end

      let(:valid_attributes) {
        {
          code: FFaker::Name.unique.name,
          name: FFaker::Name.name,
          description: FFaker::Name.name
        }
      }

      let(:invalid_attributes) {
        {
          code: nil,
          name: nil,
          description: nil
        }
      }

      describe 'GET #index' do
        it 'returns all maintenance' do
          3.times { create(:maintenance_category) }
          get :index
          result = JSON(response.body)
          expect(result['data'].count).to eq 3
        end
      end

      describe 'POST #create' do
        context 'with valid params' do
          it 'creates a new MaintenanceCategory' do
            expect {
              post :create, params: { maintenance_category: valid_attributes }, format: :json
            }.to change(MaintenanceCategory, :count).by(1)
          end
          it 'returns a success message' do
            post :create, params: { maintenance_category: valid_attributes }, format: :json
            result = JSON(response.body)
            expect(result['message']).to eq('Maintenance category saved successfully !')
          end
        end
        context 'with invalid params' do
          it 'returns error message(s)' do
            post :create, params: { maintenance_category: invalid_attributes }, format: :json
            result = JSON(response.body)
            expect(result['code'].count).to be > 0
          end
        end
      end

      describe 'PUT #update' do
        context 'with valid params' do
          it 'updates the requested maintenance category' do
            mc = create(:maintenance_category)
            put :update, params: { id: mc.id, maintenance_category: { code: 'BD' } }, format: :json
            mc.reload
            expect(mc.code).to eq('BD')
          end

          it 'returns a success message' do
            mc = create(:maintenance_category)
            put :update, params: { id: mc.id, maintenance_category: { code: 'BD' } }, format: :json
            result = JSON(response.body)
            expect(result['message']).to eq('Maintenance category updated successfully !')
          end
        end
        context 'with invalid params' do
          it 'returns error message(s)' do
            mc = create(:maintenance_category)
            put :update, params: { id: mc.id, maintenance_category: { code: nil } }, format: :json
            result = JSON(response.body)
            expect(result['code'].count).to be > 0
          end
        end
      end
    end
  end
end
