require 'rails_helper'

module Mks
  module Maintenance
    RSpec.describe MaintenanceStatusesController, type: :controller do
      routes { Mks::Maintenance::Engine.routes }

      before(:each) do
        u = create(:user)
        token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
        request.headers['Authorization'] = "Bearer #{token}"
      end

      let(:valid_attributes) {
        {
          code: FFaker::Name.unique.name,
          name: FFaker::Name.name,
          description: FFaker::Name.name
        }
      }

      let(:invalid_attributes) {
        {
          code: nil,
          name: nil,
          description: nil
        }
      }

      describe 'GET #index' do
        it 'returns all maintenance statuses' do
          3.times { create(:maintenance_status) }
          get :index
          result = JSON(response.body)
          expect(result['data'].count).to eq 3
        end
      end

      describe 'POST #create' do
        context 'with valid params' do
          it 'creates a new MaintenanceType' do
            expect {
              post :create, params: { maintenance_status: valid_attributes }, format: :json
            }.to change(MaintenanceStatus, :count).by(1)
          end
          it 'returns a success message' do
            post :create, params: { maintenance_status: valid_attributes }, format: :json
            result = JSON(response.body)
            expect(result['message']).to eq('Maintenance status saved successfully !')
          end
        end
        context 'with invalid params' do
          it 'returns error message(s)' do
            post :create, params: { maintenance_status: invalid_attributes }, format: :json
            result = JSON(response.body)
            expect(result['code'].count).to be > 0
          end
        end
      end

      describe 'PUT #update' do
        context 'with valid params' do
          it 'updates the requested maintenance status' do
            ms = create(:maintenance_status)
            put :update, params: { id: ms.id, maintenance_status: { code: 'STARTED' } }, format: :json
            ms.reload
            expect(ms.code).to eq('STARTED')
          end

          it 'returns a success message' do
            ms = create(:maintenance_status)
            put :update, params: { id: ms.id, maintenance_status: { code: 'STARTED' } }, format: :json
            result = JSON(response.body)
            expect(result['message']).to eq('Maintenance status updated successfully !')
          end
        end
        context 'with invalid params' do
          it 'returns error message(s)' do
            ms = create(:maintenance_status)
            put :update, params: { id: ms.id, maintenance_status: { code: nil } }, format: :json
            result = JSON(response.body)
            expect(result['code'].count).to be > 0
          end
        end
      end
    end
  end
end
