require 'rails_helper'

module Mks
  module Maintenance
    RSpec.describe MaintenanceTypesController, type: :controller do
      routes { Mks::Maintenance::Engine.routes }

      before(:each) do
        u = create(:user)
        token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
        request.headers['Authorization'] = "Bearer #{token}"
      end

      let(:valid_attributes) {
        {
          code: FFaker::Name.unique.name,
          name: FFaker::Name.name,
          description: FFaker::Name.name,
          maintenance_category_id: create(:maintenance_category).id
        }
      }

      let(:invalid_attributes) {
        {
          code: nil,
          name: nil,
          description: nil,
          maintenance_category_id: create(:maintenance_category).id
        }
      }

      describe 'GET #index' do
        it 'returns all maintenance types' do
          3.times { create(:maintenance_type) }
          get :index
          result = JSON(response.body)
          expect(result['data'].count).to eq 3
        end
      end

      describe 'POST #create' do
        context 'with valid params' do
          it 'creates a new MaintenanceType' do
            expect {
              post :create, params: { maintenance_type: valid_attributes }, format: :json
            }.to change(MaintenanceType, :count).by(1)
          end
          it 'returns a success message' do
            post :create, params: { maintenance_type: valid_attributes }, format: :json
            result = JSON(response.body)
            expect(result['message']).to eq('Maintenance type saved successfully !')
          end
        end
        context 'with invalid params' do
          it 'returns error message(s)' do
            post :create, params: { maintenance_type: invalid_attributes }, format: :json
            result = JSON(response.body)
            expect(result['code'].count).to be > 0
          end
        end
      end

      describe 'PUT #update' do
        context 'with valid params' do
          it 'updates the requested maintenance type' do
            mt = create(:maintenance_type)
            put :update, params: { id: mt.id, maintenance_type: { code: 'RS' } }, format: :json
            mt.reload
            expect(mt.code).to eq('RS')
          end

          it 'returns a success message' do
            mt = create(:maintenance_type)
            put :update, params: { id: mt.id, maintenance_type: { code: 'RS' } }, format: :json
            result = JSON(response.body)
            expect(result['message']).to eq('Maintenance type updated successfully !')
          end
        end
        context 'with invalid params' do
          it 'returns error message(s)' do
            mt = create(:maintenance_type)
            put :update, params: { id: mt.id, maintenance_type: { code: nil } }, format: :json
            result = JSON(response.body)
            expect(result['code'].count).to be > 0
          end
        end
      end
    end
  end
end
