require 'rails_helper'

module Mks
  module Maintenance
    RSpec.describe ScheduleUnitsController, type: :controller do
      routes { Mks::Maintenance::Engine.routes }

      before(:each) do
        u = create(:user)
        token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
        request.headers['Authorization'] = "Bearer #{token}"
      end

      let(:valid_attributes) {
        {
          code: FFaker::Name.unique.name,
          name: FFaker::Name.name,
          description: FFaker::Name.name
        }
      }

      let(:invalid_attributes) {
        {
          code: nil,
          name: nil,
          description: nil
        }
      }

      describe 'GET #index' do
        it 'returns all schedule units' do
          3.times { create(:schedule_unit) }
          get :index
          result = JSON(response.body)
          expect(result['data'].count).to eq 3
        end
      end

      describe 'POST #create' do
        context 'with valid params' do
          it 'creates a new ScheduleUnit' do
            expect {
              post :create, params: { schedule_unit: valid_attributes }, format: :json
            }.to change(ScheduleUnit, :count).by(1)
          end
          it 'returns a success message' do
            post :create, params: { schedule_unit: valid_attributes }, format: :json
            result = JSON(response.body)
            expect(result['message']).to eq('Schedule unit saved successfully !')
          end
        end
        context 'with invalid params' do
          it 'returns error message(s)' do
            post :create, params: { schedule_unit: invalid_attributes }, format: :json
            result = JSON(response.body)
            expect(result['code'].count).to be > 0
          end
        end
      end

      describe 'PUT #update' do
        context 'with valid params' do
          it 'updates the requested schedule unit' do
            su = create(:schedule_unit)
            put :update, params: { id: su.id, schedule_unit: { code: 'DAY' } }, format: :json
            su.reload
            expect(su.code).to eq('DAY')
          end

          it 'returns a success message' do
            su = create(:schedule_unit)
            put :update, params: { id: su.id, schedule_unit: { code: 'DAY' } }, format: :json
            result = JSON(response.body)
            expect(result['message']).to eq('Schedule unit updated successfully !')
          end
        end
        context 'with invalid params' do
          it 'returns error message(s)' do
            su = create(:schedule_unit)
            put :update, params: { id: su.id, schedule_unit: { code: nil } }, format: :json
            result = JSON(response.body)
            expect(result['code'].count).to be > 0
          end
        end
      end
    end
  end
end
